package com.xh.common.util;

import android.text.TextUtils;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class XhDateUtil {
    private static Map<String, DateFormat> FormatterPool = new HashMap<String, DateFormat>();

    public static Date getCurrentTime() {
        return Calendar.getInstance().getTime();
    }

    public static String getCurrentTime(String pattern) {
        return formatTime(pattern, getCurrentTime());
    }

    private static DateFormat getFormatter(String pattern) {
        DateFormat df = FormatterPool.get(pattern);
        if (df == null) {
            df = new SimpleDateFormat(pattern);
            FormatterPool.put(pattern, df);
        }
        return df;
    }

    public static String formatTime(String pattern, long timeInMillis) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(timeInMillis);
        return getFormatter(pattern).format(calendar.getTime());
    }

    public static String formatTime(String pattern, Date time) {
        return getFormatter(pattern).format(time);
    }

    public static String formatTime(String pattern, String value) throws ParseException {
        Date date = parseTime(value, pattern);
        return getFormatter(pattern).format(date);
    }

    public static Date parseTime(String value, String pattern) throws ParseException {
        return getFormatter(pattern).parse(value);
    }

    public static long parseTimeToLong(String value, String pattern) throws ParseException {
        return getFormatter(pattern).parse(value).getTime();
    }

    public static String getWeekDate(long timeValue) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(timeValue);

        int weekDay = calendar.get(Calendar.DAY_OF_WEEK);
        String weekDayStr = "星期";
        if (weekDay == 1) {
            weekDayStr += "日";
        } else if (weekDay == 2) {
            weekDayStr += "一";
        } else if (weekDay == 3) {
            weekDayStr += "二";
        } else if (weekDay == 4) {
            weekDayStr += "三";
        } else if (weekDay == 5) {
            weekDayStr += "四";
        } else if (weekDay == 6) {
            weekDayStr += "五";
        } else if (weekDay == 7) {
            weekDayStr += "六";
        }
        return weekDayStr;
    }

    public static List<String> getWeekDates(String dateStr) {
        List<String> weekDateList = new ArrayList<String>();
        try {
            Date date = parseTime(dateStr, "yyyy-MM-dd");
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);

            int weekDay = calendar.get(Calendar.DAY_OF_WEEK);
            if (weekDay == 1) {
                calendar.add(Calendar.DATE, -6);
            } else if (weekDay == 3) {
                calendar.add(Calendar.DATE, -1);
            } else if (weekDay == 4) {
                calendar.add(Calendar.DATE, -2);
            } else if (weekDay == 5) {
                calendar.add(Calendar.DATE, -3);
            } else if (weekDay == 6) {
                calendar.add(Calendar.DATE, -4);
            } else if (weekDay == 7) {
                calendar.add(Calendar.DATE, -5);
            }

            for (int i = 0; i < 7; i++) {
                weekDateList.add(formatTime("yyyy-MM-dd", calendar.getTime()));
                calendar.add(Calendar.DATE, 1);
            }
        } catch (ParseException e) {
            e.printStackTrace();
            weekDateList.clear();
        }
        return weekDateList;
    }

    /**
     * 获取普通消息时间
     *
     * @param timeValue
     * @return
     */
    public static String getMessageTime(String timeValue) {
        if (TextUtils.isEmpty(timeValue)) {
            return "";
        }
        Calendar calendar = Calendar.getInstance();
        Calendar calendar1 = Calendar.getInstance();
        calendar1.setTimeInMillis(Timestamp.valueOf(timeValue).getTime());
        if (formatTime("yyyy-MM-dd", calendar.getTimeInMillis()).equals(formatTime("yyyy-MM-dd", calendar1.getTimeInMillis()))) {
            return "今天 " + formatTime("HH:mm", calendar1.getTimeInMillis());
        }
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        if (formatTime("yyyy-MM-dd", calendar.getTimeInMillis()).equals(formatTime("yyyy-MM-dd", calendar1.getTimeInMillis()))) {
            return "昨天 " + formatTime("HH:mm", calendar1.getTimeInMillis());
        }
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        if (formatTime("yyyy-MM-dd", calendar.getTimeInMillis()).equals(formatTime("yyyy-MM-dd", calendar1.getTimeInMillis()))) {
            return "前天 " + formatTime("HH:mm", calendar1.getTimeInMillis());
        }
        if (calendar.get(Calendar.YEAR) == calendar1.get(Calendar.YEAR)) {
            return formatTime("MM月dd日", calendar1.getTimeInMillis());
        }
        return formatTime("yyyy年MM月dd日", calendar1.getTimeInMillis());
    }

    /**
     * 获取聊天时间
     *
     * @param timesamp
     * @return
     */
    public static String getChatTime(long timesamp) {
        Calendar calendar = Calendar.getInstance();
        Calendar calendar1 = Calendar.getInstance();
        calendar1.setTimeInMillis(timesamp);
        if (formatTime("yyyy-MM-dd", calendar.getTimeInMillis()).equals(formatTime("yyyy-MM-dd", calendar1.getTimeInMillis()))) {
            return "今天 " + formatTime("HH:mm", calendar1.getTimeInMillis());
        }
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        if (formatTime("yyyy-MM-dd", calendar.getTimeInMillis()).equals(formatTime("yyyy-MM-dd", calendar1.getTimeInMillis()))) {
            return "昨天 " + formatTime("HH:mm", calendar1.getTimeInMillis());
        }
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        if (formatTime("yyyy-MM-dd", calendar.getTimeInMillis()).equals(formatTime("yyyy-MM-dd", calendar1.getTimeInMillis()))) {
            return "前天 " + formatTime("HH:mm", calendar1.getTimeInMillis());
        }
        return formatTime("yyyy-MM-dd HH:mm", calendar1.getTimeInMillis());
    }

    /**
     * 计算宝宝年龄 *岁*个月*天
     */
    public static String getAge(String birth){
        Date currentTime = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String dateString = formatter.format(currentTime);
        if (birth == null || birth.equals(""))
            return "0";
        if (dateString == null || dateString.equals(""))
            return "0";
        // 转换为标准时间
        SimpleDateFormat myFormatter = new SimpleDateFormat("yyyy-MM-dd");
        java.util.Date date = null;
        java.util.Date mydate = null;
        try {
            date = myFormatter.parse(birth);
            mydate = myFormatter.parse(dateString);
        } catch (Exception e) {
        }
        long totalDays = ( mydate.getTime()-date.getTime()) / (24 * 60 * 60 * 1000);
        long year=totalDays/365;
        long month=(totalDays-365L*year)/30;
        long day=totalDays-365L*year-30l*month;
        return year+"岁 "+month+"个月 "+day+"天";

    }
}
